@@ -45,6 +45,20 @@ pub(crate) const MAX_HTLCS: u16 = 483;
 pub(super) const HTLC_SUCCESS_TX_WEIGHT: u64 = 703;
 pub(super) const HTLC_TIMEOUT_TX_WEIGHT: u64 = 663;
 
+/// Gets the weight for an HTLC-Success transaction.
+#[inline]
+pub fn htlc_success_tx_weight(opt_anchors: bool) -> u64 {
+	const HTLC_SUCCESS_ANCHOR_TX_WEIGHT: u64 = 706;
+	if opt_anchors { HTLC_SUCCESS_ANCHOR_TX_WEIGHT } else { HTLC_SUCCESS_TX_WEIGHT }
+}
+
+/// Gets the weight for an HTLC-Timeout transaction.
+#[inline]
+pub fn htlc_timeout_tx_weight(opt_anchors: bool) -> u64 {
+	const HTLC_TIMEOUT_ANCHOR_TX_WEIGHT: u64 = 666;
+	if opt_anchors { HTLC_TIMEOUT_ANCHOR_TX_WEIGHT } else { HTLC_TIMEOUT_TX_WEIGHT }
+}
+
 #[derive(PartialEq)]
 pub(crate) enum HTLCType {
 	AcceptedHTLC,
@@ -482,10 +496,10 @@ impl_writeable_tlv_based!(HTLCOutputInCommitment, {
 });
 
 #[inline]
-pub(crate) fn get_htlc_redeemscript_with_explicit_keys(htlc: &HTLCOutputInCommitment, broadcaster_htlc_key: &PublicKey, countersignatory_htlc_key: &PublicKey, revocation_key: &PublicKey) -> Script {
+pub(crate) fn get_htlc_redeemscript_with_explicit_keys(htlc: &HTLCOutputInCommitment, opt_anchors: bool, broadcaster_htlc_key: &PublicKey, countersignatory_htlc_key: &PublicKey, revocation_key: &PublicKey) -> Script {
 	let payment_hash160 = Ripemd160::hash(&htlc.payment_hash.0[..]).into_inner();
 	if htlc.offered {
-		Builder::new().push_opcode(opcodes::all::OP_DUP)
+		let mut bldr = Builder::new().push_opcode(opcodes::all::OP_DUP)
 		              .push_opcode(opcodes::all::OP_HASH160)
 		              .push_slice(&PubkeyHash::hash(&revocation_key.serialize())[..])
 		              .push_opcode(opcodes::all::OP_EQUAL)
@@ -509,11 +523,16 @@ pub(crate) fn get_htlc_redeemscript_with_explicit_keys(htlc: &HTLCOutputInCommit
 		              .push_slice(&payment_hash160)
 		              .push_opcode(opcodes::all::OP_EQUALVERIFY)
 		              .push_opcode(opcodes::all::OP_CHECKSIG)
-		              .push_opcode(opcodes::all::OP_ENDIF)
-		              .push_opcode(opcodes::all::OP_ENDIF)
-		              .into_script()
+		              .push_opcode(opcodes::all::OP_ENDIF);
+		if opt_anchors {
+			bldr = bldr.push_opcode(opcodes::all::OP_PUSHNUM_1)
+				.push_opcode(opcodes::all::OP_CSV)
+				.push_opcode(opcodes::all::OP_DROP);
+		}
+		bldr.push_opcode(opcodes::all::OP_ENDIF)
+			.into_script()
 	} else {
-		Builder::new().push_opcode(opcodes::all::OP_DUP)
+			let mut bldr = Builder::new().push_opcode(opcodes::all::OP_DUP)
 		              .push_opcode(opcodes::all::OP_HASH160)
 		              .push_slice(&PubkeyHash::hash(&revocation_key.serialize())[..])
 		              .push_opcode(opcodes::all::OP_EQUAL)
@@ -540,17 +559,22 @@ pub(crate) fn get_htlc_redeemscript_with_explicit_keys(htlc: &HTLCOutputInCommit
 		              .push_opcode(opcodes::all::OP_CLTV)
 		              .push_opcode(opcodes::all::OP_DROP)
 		              .push_opcode(opcodes::all::OP_CHECKSIG)
-		              .push_opcode(opcodes::all::OP_ENDIF)
-		              .push_opcode(opcodes::all::OP_ENDIF)
-		              .into_script()
+		              .push_opcode(opcodes::all::OP_ENDIF);
+		if opt_anchors {
+			bldr = bldr.push_opcode(opcodes::all::OP_PUSHNUM_1)
+				.push_opcode(opcodes::all::OP_CSV)
+				.push_opcode(opcodes::all::OP_DROP);
+		}
+		bldr.push_opcode(opcodes::all::OP_ENDIF)
+			.into_script()
 	}
 }
 
 /// Gets the witness redeemscript for an HTLC output in a commitment transaction. Note that htlc
 /// does not need to have its previous_output_index filled.
 #[inline]
-pub fn get_htlc_redeemscript(htlc: &HTLCOutputInCommitment, keys: &TxCreationKeys) -> Script {
-	get_htlc_redeemscript_with_explicit_keys(htlc, &keys.broadcaster_htlc_key, &keys.countersignatory_htlc_key, &keys.revocation_key)
+pub fn get_htlc_redeemscript(htlc: &HTLCOutputInCommitment, opt_anchors: bool, keys: &TxCreationKeys) -> Script {
+	get_htlc_redeemscript_with_explicit_keys(htlc, opt_anchors, &keys.broadcaster_htlc_key, &keys.countersignatory_htlc_key, &keys.revocation_key)
 }
 
 /// Gets the redeemscript for a funding output from the two funding public keys.
@@ -576,7 +600,7 @@ pub fn make_funding_redeemscript(broadcaster: &PublicKey, countersignatory: &Pub
 ///
 /// Panics if htlc.transaction_output_index.is_none() (as such HTLCs do not appear in the
 /// commitment transaction).
-pub fn build_htlc_transaction(commitment_txid: &Txid, feerate_per_kw: u32, contest_delay: u16, htlc: &HTLCOutputInCommitment, broadcaster_delayed_payment_key: &PublicKey, revocation_key: &PublicKey) -> Transaction {
+pub fn build_htlc_transaction(commitment_txid: &Txid, feerate_per_kw: u32, contest_delay: u16, htlc: &HTLCOutputInCommitment, opt_anchors: bool, broadcaster_delayed_payment_key: &PublicKey, revocation_key: &PublicKey) -> Transaction {
 	let mut txins: Vec<TxIn> = Vec::new();
 	txins.push(TxIn {
 		previous_output: OutPoint {
@@ -584,15 +608,16 @@ pub fn build_htlc_transaction(commitment_txid: &Txid, feerate_per_kw: u32, conte
 			vout: htlc.transaction_output_index.expect("Can't build an HTLC transaction for a dust output"),
 		},
 		script_sig: Script::new(),
-		sequence: 0,
+		sequence: if opt_anchors { 1 } else { 0 },
 		witness: Vec::new(),
 	});
 
-	let total_fee = if htlc.offered {
-			feerate_per_kw as u64 * HTLC_TIMEOUT_TX_WEIGHT / 1000
-		} else {
-			feerate_per_kw as u64 * HTLC_SUCCESS_TX_WEIGHT / 1000
-		};
+	let weight = if htlc.offered {
+		htlc_timeout_tx_weight(opt_anchors)
+	} else {
+		htlc_success_tx_weight(opt_anchors)
+	};
+	let total_fee = feerate_per_kw as u64 * weight / 1000;
 
 	let mut txouts: Vec<TxOut> = Vec::new();
 	txouts.push(TxOut {
@@ -608,6 +633,17 @@ pub fn build_htlc_transaction(commitment_txid: &Txid, feerate_per_kw: u32, conte
 	}
 }
 
+/// Gets the witnessScript for the to_remote output when anchors are enabled.
+#[inline]
+pub(crate) fn get_to_countersignatory_with_anchors_redeemscript(payment_point: &PublicKey) -> Script {
+	Builder::new()
+		.push_slice(&payment_point.serialize()[..])
+		.push_opcode(opcodes::all::OP_CHECKSIGVERIFY)
+		.push_int(1)
+		.push_opcode(opcodes::all::OP_CSV)
+		.into_script()
+}
+
 /// Gets the witnessScript for an anchor output from the funding public key.
 /// The witness in the spending input must be:
 /// <BIP 143 funding_signature>
@@ -615,7 +651,7 @@ pub fn build_htlc_transaction(commitment_txid: &Txid, feerate_per_kw: u32, conte
 /// <>
 /// (empty vector required to satisfy compliance with MINIMALIF-standard rule)
 #[inline]
-pub(crate) fn get_anchor_redeemscript(funding_pubkey: &PublicKey) -> Script {
+pub fn get_anchor_redeemscript(funding_pubkey: &PublicKey) -> Script {
 	Builder::new().push_slice(&funding_pubkey.serialize()[..])
 		.push_opcode(opcodes::all::OP_CHECKSIG)
 		.push_opcode(opcodes::all::OP_IFDUP)
@@ -645,6 +681,8 @@ pub struct ChannelTransactionParameters {
 	pub counterparty_parameters: Option<CounterpartyChannelTransactionParameters>,
 	/// The late-bound funding outpoint
 	pub funding_outpoint: Option<chain::transaction::OutPoint>,
+	/// Are anchors used for this channel.  Boolean is serialization backwards-compatible
+	pub opt_anchors: Option<()>
 }
 
 /// Late-bound per-channel counterparty data used to build transactions.
@@ -698,6 +736,7 @@ impl_writeable_tlv_based!(ChannelTransactionParameters, {
 	(4, is_outbound_from_holder, required),
 	(6, counterparty_parameters, option),
 	(8, funding_outpoint, option),
+	(10, opt_anchors, option),
 });
 
 /// Static channel fields used to build transactions given per-commitment fields, organized by
@@ -750,6 +789,11 @@ impl<'a> DirectedChannelTransactionParameters<'a> {
 	pub fn funding_outpoint(&self) -> OutPoint {
 		self.inner.funding_outpoint.unwrap().into_bitcoin_outpoint()
 	}
+
+	/// Whether to use anchors for this channel
+	pub fn opt_anchors(&self) -> bool {
+		self.inner.opt_anchors.is_some()
+	}
 }
 
 /// Information needed to build and sign a holder's commitment transaction.
@@ -813,7 +857,8 @@ impl HolderCommitmentTransaction {
 			holder_selected_contest_delay: 0,
 			is_outbound_from_holder: false,
 			counterparty_parameters: Some(CounterpartyChannelTransactionParameters { pubkeys: channel_pubkeys.clone(), selected_contest_delay: 0 }),
-			funding_outpoint: Some(chain::transaction::OutPoint { txid: Default::default(), index: 0 })
+			funding_outpoint: Some(chain::transaction::OutPoint { txid: Default::default(), index: 0 }),
+			opt_anchors: None
 		};
 		let mut htlcs_with_aux: Vec<(_, ())> = Vec::new();
 		let inner = CommitmentTransaction::new_with_auxiliary_htlc_data(0, 0, 0, false, dummy_key.clone(), dummy_key.clone(), keys, 0, &mut htlcs_with_aux, &channel_parameters.as_counterparty_broadcastable());
@@ -895,6 +940,7 @@ impl BuiltCommitmentTransaction {
 ///
 /// This class can be used inside a signer implementation to generate a signature given the relevant
 /// secret key.
+#[derive(Clone, Hash, PartialEq)]
 pub struct ClosingTransaction {
 	to_holder_value_sat: u64,
 	to_counterparty_value_sat: u64,
@@ -1130,7 +1176,11 @@ impl CommitmentTransaction {
 		let mut txouts: Vec<(TxOut, Option<&mut HTLCOutputInCommitment>)> = Vec::new();
 
 		if to_countersignatory_value_sat > 0 {
-			let script = script_for_p2wpkh(&countersignatory_pubkeys.payment_point);
+			let script = if opt_anchors {
+			    get_to_countersignatory_with_anchors_redeemscript(&countersignatory_pubkeys.payment_point).to_v0_p2wsh()
+			} else {
+			    get_p2wpkh_redeemscript(&countersignatory_pubkeys.payment_point)
+			};
 			txouts.push((
 				TxOut {
 					script_pubkey: script.clone(),
@@ -1181,7 +1231,7 @@ impl CommitmentTransaction {
 
 		let mut htlcs = Vec::with_capacity(htlcs_with_aux.len());
 		for (htlc, _) in htlcs_with_aux {
-			let script = chan_utils::get_htlc_redeemscript(&htlc, &keys);
+			let script = chan_utils::get_htlc_redeemscript(&htlc, opt_anchors, &keys);
 			let txout = TxOut {
 				script_pubkey: script.to_v0_p2wsh(),
 				value: htlc.amount_msat / 1000,
@@ -1335,6 +1385,11 @@ impl<'a> TrustedCommitmentTransaction<'a> {
 		&self.inner.keys
 	}
 
+	/// Should anchors be used.
+	pub fn opt_anchors(&self) -> bool {
+		self.opt_anchors.is_some()
+	}
+
 	/// Get a signature for each HTLC which was included in the commitment transaction (ie for
 	/// which HTLCOutputInCommitment::transaction_output_index.is_some()).
 	///
@@ -1348,9 +1403,9 @@ impl<'a> TrustedCommitmentTransaction<'a> {
 
 		for this_htlc in inner.htlcs.iter() {
 			assert!(this_htlc.transaction_output_index.is_some());
-			let htlc_tx = build_htlc_transaction(&txid, inner.feerate_per_kw, channel_parameters.contest_delay(), &this_htlc, &keys.broadcaster_delayed_payment_key, &keys.revocation_key);
+			let htlc_tx = build_htlc_transaction(&txid, inner.feerate_per_kw, channel_parameters.contest_delay(), &this_htlc, self.opt_anchors(), &keys.broadcaster_delayed_payment_key, &keys.revocation_key);
 
-			let htlc_redeemscript = get_htlc_redeemscript_with_explicit_keys(&this_htlc, &keys.broadcaster_htlc_key, &keys.countersignatory_htlc_key, &keys.revocation_key);
+			let htlc_redeemscript = get_htlc_redeemscript_with_explicit_keys(&this_htlc, self.opt_anchors(), &keys.broadcaster_htlc_key, &keys.countersignatory_htlc_key, &keys.revocation_key);
 
 			let sighash = hash_to_message!(&bip143::SigHashCache::new(&htlc_tx).signature_hash(0, &htlc_redeemscript, this_htlc.amount_msat / 1000, SigHashType::All)[..]);
 			ret.push(secp_ctx.sign(&sighash, &holder_htlc_key));
@@ -1370,9 +1425,9 @@ impl<'a> TrustedCommitmentTransaction<'a> {
 		// Further, we should never be provided the preimage for an HTLC-Timeout transaction.
 		if  this_htlc.offered && preimage.is_some() { unreachable!(); }
 
-		let mut htlc_tx = build_htlc_transaction(&txid, inner.feerate_per_kw, channel_parameters.contest_delay(), &this_htlc, &keys.broadcaster_delayed_payment_key, &keys.revocation_key);
+		let mut htlc_tx = build_htlc_transaction(&txid, inner.feerate_per_kw, channel_parameters.contest_delay(), &this_htlc, self.opt_anchors(), &keys.broadcaster_delayed_payment_key, &keys.revocation_key);
 
-		let htlc_redeemscript = get_htlc_redeemscript_with_explicit_keys(&this_htlc, &keys.broadcaster_htlc_key, &keys.countersignatory_htlc_key, &keys.revocation_key);
+		let htlc_redeemscript = get_htlc_redeemscript_with_explicit_keys(&this_htlc, self.opt_anchors(), &keys.broadcaster_htlc_key, &keys.countersignatory_htlc_key, &keys.revocation_key);
 
 		// First push the multisig dummy, note that due to BIP147 (NULLDUMMY) it must be a zero-length element.
 		htlc_tx.input[0].witness.push(Vec::new());
@@ -1424,7 +1479,7 @@ pub fn get_commitment_transaction_number_obscure_factor(
 		| ((res[31] as u64) << 0 * 8)
 }
 
-fn script_for_p2wpkh(key: &PublicKey) -> Script {
+fn get_p2wpkh_redeemscript(key: &PublicKey) -> Script {
 	Builder::new().push_opcode(opcodes::all::OP_PUSHBYTES_0)
 		.push_slice(&WPubkeyHash::hash(&key.serialize())[..])
 		.into_script()
@@ -1435,12 +1490,13 @@ mod tests {
 	use super::CounterpartyCommitmentSecrets;
 	use ::{hex, chain};
 	use prelude::*;
-	use ln::chan_utils::{CommitmentTransaction, TxCreationKeys, ChannelTransactionParameters, CounterpartyChannelTransactionParameters, HTLCOutputInCommitment};
+	use ln::chan_utils::{get_htlc_redeemscript, get_to_countersignatory_with_anchors_redeemscript, get_p2wpkh_redeemscript, CommitmentTransaction, TxCreationKeys, ChannelTransactionParameters, CounterpartyChannelTransactionParameters, HTLCOutputInCommitment};
 	use bitcoin::secp256k1::{PublicKey, SecretKey, Secp256k1};
 	use util::test_utils;
 	use chain::keysinterface::{KeysInterface, BaseSign};
 	use bitcoin::Network;
 	use ln::PaymentHash;
+	use bitcoin::hashes::hex::ToHex;
 
 	#[test]
 	fn test_anchors() {
@@ -1458,12 +1514,13 @@ mod tests {
 		let holder_pubkeys = signer.pubkeys();
 		let counterparty_pubkeys = counterparty_signer.pubkeys();
 		let keys = TxCreationKeys::derive_new(&secp_ctx, &per_commitment_point, delayed_payment_base, htlc_basepoint, &counterparty_pubkeys.revocation_basepoint, &counterparty_pubkeys.htlc_basepoint).unwrap();
-		let channel_parameters = ChannelTransactionParameters {
+		let mut channel_parameters = ChannelTransactionParameters {
 			holder_pubkeys: holder_pubkeys.clone(),
 			holder_selected_contest_delay: 0,
 			is_outbound_from_holder: false,
 			counterparty_parameters: Some(CounterpartyChannelTransactionParameters { pubkeys: counterparty_pubkeys.clone(), selected_contest_delay: 0 }),
-			funding_outpoint: Some(chain::transaction::OutPoint { txid: Default::default(), index: 0 })
+			funding_outpoint: Some(chain::transaction::OutPoint { txid: Default::default(), index: 0 }),
+			opt_anchors: None
 		};
 
 		let mut htlcs_with_aux: Vec<(_, ())> = Vec::new();
@@ -1478,6 +1535,7 @@ mod tests {
 			&mut htlcs_with_aux, &channel_parameters.as_holder_broadcastable()
 		);
 		assert_eq!(tx.built.transaction.output.len(), 2);
+		assert_eq!(tx.built.transaction.output[1].script_pubkey, get_p2wpkh_redeemscript(&counterparty_pubkeys.payment_point));
 
 		// Generate broadcaster and counterparty outputs as well as two anchors
 		let tx = CommitmentTransaction::new_with_auxiliary_htlc_data(
@@ -1489,6 +1547,7 @@ mod tests {
 			&mut htlcs_with_aux, &channel_parameters.as_holder_broadcastable()
 		);
 		assert_eq!(tx.built.transaction.output.len(), 4);
+		assert_eq!(tx.built.transaction.output[3].script_pubkey, get_to_countersignatory_with_anchors_redeemscript(&counterparty_pubkeys.payment_point).to_v0_p2wsh());
 
 		// Generate broadcaster output and anchor
 		let tx = CommitmentTransaction::new_with_auxiliary_htlc_data(
@@ -1512,25 +1571,58 @@ mod tests {
 		);
 		assert_eq!(tx.built.transaction.output.len(), 2);
 
-		// Generate broadcaster output, an HTLC output and two anchors
-		let payment_hash = PaymentHash([42; 32]);
-		let htlc_info = HTLCOutputInCommitment {
+		let received_htlc = HTLCOutputInCommitment {
 			offered: false,
-			amount_msat: 1000000,
+			amount_msat: 400000,
 			cltv_expiry: 100,
-			payment_hash,
+			payment_hash: PaymentHash([42; 32]),
 			transaction_output_index: None,
 		};
 
+		let offered_htlc = HTLCOutputInCommitment {
+			offered: true,
+			amount_msat: 600000,
+			cltv_expiry: 100,
+			payment_hash: PaymentHash([43; 32]),
+			transaction_output_index: None,
+		};
+
+		// Generate broadcaster output and received and offered HTLC outputs,  w/o anchors
+		let tx = CommitmentTransaction::new_with_auxiliary_htlc_data(
+			0, 3000, 0,
+			false,
+			holder_pubkeys.funding_pubkey,
+			counterparty_pubkeys.funding_pubkey,
+			keys.clone(), 1,
+			&mut vec![(received_htlc.clone(), ()), (offered_htlc.clone(), ())],
+			&channel_parameters.as_holder_broadcastable()
+		);
+		assert_eq!(tx.built.transaction.output.len(), 3);
+		assert_eq!(tx.built.transaction.output[0].script_pubkey, get_htlc_redeemscript(&received_htlc, false, &keys).to_v0_p2wsh());
+		assert_eq!(tx.built.transaction.output[1].script_pubkey, get_htlc_redeemscript(&offered_htlc, false, &keys).to_v0_p2wsh());
+		assert_eq!(get_htlc_redeemscript(&received_htlc, false, &keys).to_v0_p2wsh().to_hex(),
+				   "002085cf52e41ba7c099a39df504e7b61f6de122971ceb53b06731876eaeb85e8dc5");
+		assert_eq!(get_htlc_redeemscript(&offered_htlc, false, &keys).to_v0_p2wsh().to_hex(),
+				   "002049f0736bb335c61a04d2623a24df878a7592a3c51fa7258d41b2c85318265e73");
+
+		// Generate broadcaster output and received and offered HTLC outputs,  with anchors
+		channel_parameters.opt_anchors = Some(());
 		let tx = CommitmentTransaction::new_with_auxiliary_htlc_data(
 			0, 3000, 0,
 			true,
 			holder_pubkeys.funding_pubkey,
 			counterparty_pubkeys.funding_pubkey,
 			keys.clone(), 1,
-			&mut vec![(htlc_info, ())], &channel_parameters.as_holder_broadcastable()
+			&mut vec![(received_htlc.clone(), ()), (offered_htlc.clone(), ())],
+			&channel_parameters.as_holder_broadcastable()
 		);
-		assert_eq!(tx.built.transaction.output.len(), 4);
+		assert_eq!(tx.built.transaction.output.len(), 5);
+		assert_eq!(tx.built.transaction.output[2].script_pubkey, get_htlc_redeemscript(&received_htlc, true, &keys).to_v0_p2wsh());
+		assert_eq!(tx.built.transaction.output[3].script_pubkey, get_htlc_redeemscript(&offered_htlc, true, &keys).to_v0_p2wsh());
+		assert_eq!(get_htlc_redeemscript(&received_htlc, true, &keys).to_v0_p2wsh().to_hex(),
+				   "002067114123af3f95405bae4fd930fc95de03e3c86baaee8b2dd29b43dd26cf613c");
+		assert_eq!(get_htlc_redeemscript(&offered_htlc, true, &keys).to_v0_p2wsh().to_hex(),
+				   "0020a06e3b0d4fcf704f2b9c41e16a70099e39989466c3142b8573a1154542f28f57");
 	}
 
 	#[test]
