@@ -22,7 +22,7 @@
 //! [BOLT #9]: https://github.com/lightningnetwork/lightning-rfc/blob/master/09-features.md
 //! [messages]: crate::ln::msgs
 
-use io;
+use {io, io_extras};
 use prelude::*;
 use core::{cmp, fmt};
 use core::hash::{Hash, Hasher};
@@ -126,6 +126,10 @@ mod sealed {
 			,
 			// Byte 3
 			,
+			// Byte 4
+			,
+			// Byte 5
+			,
 		],
 		optional_features: [
 			// Byte 0
@@ -136,6 +140,10 @@ mod sealed {
 			BasicMPP,
 			// Byte 3
 			ShutdownAnySegwit,
+			// Byte 4
+			,
+			// Byte 5
+			ChannelType,
 		],
 	});
 	define_context!(NodeContext {
@@ -167,7 +175,7 @@ mod sealed {
 			// Byte 4
 			,
 			// Byte 5
-			,
+			ChannelType,
 			// Byte 6
 			Keysend,
 		],
@@ -194,12 +202,36 @@ mod sealed {
 			BasicMPP,
 		],
 	});
+	// This isn't a "real" feature context, and is only used in the channel_type field in an
+	// `OpenChannel` message.
+	define_context!(ChannelTypeContext {
+		required_features: [
+			// Byte 0
+			,
+			// Byte 1
+			StaticRemoteKey,
+			// Byte 2
+			,
+			// Byte 3
+			,
+		],
+		optional_features: [
+			// Byte 0
+			,
+			// Byte 1
+			,
+			// Byte 2
+			,
+			// Byte 3
+			,
+		],
+	});
 
 	/// Defines a feature with the given bits for the specified [`Context`]s. The generated trait is
 	/// useful for manipulating feature flags.
 	macro_rules! define_feature {
 		($odd_bit: expr, $feature: ident, [$($context: ty),+], $doc: expr, $optional_setter: ident,
-		 $required_setter: ident) => {
+		 $required_setter: ident, $supported_getter: ident) => {
 			#[doc = $doc]
 			///
 			/// See [BOLT #9] for details.
@@ -296,6 +328,11 @@ mod sealed {
 					<T as $feature>::set_required_bit(&mut self.flags);
 					self
 				}
+
+				/// Checks if this feature is supported.
+				pub fn $supported_getter(&self) -> bool {
+					<T as $feature>::supports_feature(&self.flags)
+				}
 			}
 
 			$(
@@ -307,41 +344,60 @@ mod sealed {
 					const ASSERT_ODD_BIT_PARITY: usize = (<Self as $feature>::ODD_BIT % 2) - 1;
 				}
 			)*
-
+		};
+		($odd_bit: expr, $feature: ident, [$($context: ty),+], $doc: expr, $optional_setter: ident,
+		 $required_setter: ident, $supported_getter: ident, $required_getter: ident) => {
+			define_feature!($odd_bit, $feature, [$($context),+], $doc, $optional_setter, $required_setter, $supported_getter);
+			impl <T: $feature> Features<T> {
+				/// Checks if this feature is required.
+				pub fn $required_getter(&self) -> bool {
+					<T as $feature>::requires_feature(&self.flags)
+				}
+			}
 		}
 	}
 
 	define_feature!(1, DataLossProtect, [InitContext, NodeContext],
 		"Feature flags for `option_data_loss_protect`.", set_data_loss_protect_optional,
-		set_data_loss_protect_required);
+		set_data_loss_protect_required, supports_data_loss_protect, requires_data_loss_protect);
 	// NOTE: Per Bolt #9, initial_routing_sync has no even bit.
 	define_feature!(3, InitialRoutingSync, [InitContext], "Feature flags for `initial_routing_sync`.",
-		set_initial_routing_sync_optional, set_initial_routing_sync_required);
+		set_initial_routing_sync_optional, set_initial_routing_sync_required,
+		initial_routing_sync);
 	define_feature!(5, UpfrontShutdownScript, [InitContext, NodeContext],
 		"Feature flags for `option_upfront_shutdown_script`.", set_upfront_shutdown_script_optional,
-		set_upfront_shutdown_script_required);
+		set_upfront_shutdown_script_required, supports_upfront_shutdown_script,
+		requires_upfront_shutdown_script);
 	define_feature!(7, GossipQueries, [InitContext, NodeContext],
-		"Feature flags for `gossip_queries`.", set_gossip_queries_optional, set_gossip_queries_required);
+		"Feature flags for `gossip_queries`.", set_gossip_queries_optional, set_gossip_queries_required,
+		supports_gossip_queries, requires_gossip_queries);
 	define_feature!(9, VariableLengthOnion, [InitContext, NodeContext, InvoiceContext],
 		"Feature flags for `var_onion_optin`.", set_variable_length_onion_optional,
-		set_variable_length_onion_required);
-	define_feature!(13, StaticRemoteKey, [InitContext, NodeContext],
+		set_variable_length_onion_required, supports_variable_length_onion,
+		requires_variable_length_onion);
+	define_feature!(13, StaticRemoteKey, [InitContext, NodeContext, ChannelTypeContext],
 		"Feature flags for `option_static_remotekey`.", set_static_remote_key_optional,
-		set_static_remote_key_required);
+		set_static_remote_key_required, supports_static_remote_key, requires_static_remote_key);
 	define_feature!(15, PaymentSecret, [InitContext, NodeContext, InvoiceContext],
-		"Feature flags for `payment_secret`.", set_payment_secret_optional, set_payment_secret_required);
+		"Feature flags for `payment_secret`.", set_payment_secret_optional, set_payment_secret_required,
+		supports_payment_secret, requires_payment_secret);
 	define_feature!(17, BasicMPP, [InitContext, NodeContext, InvoiceContext],
-		"Feature flags for `basic_mpp`.", set_basic_mpp_optional, set_basic_mpp_required);
+		"Feature flags for `basic_mpp`.", set_basic_mpp_optional, set_basic_mpp_required,
+		supports_basic_mpp, requires_basic_mpp);
 	define_feature!(27, ShutdownAnySegwit, [InitContext, NodeContext],
 		"Feature flags for `opt_shutdown_anysegwit`.", set_shutdown_any_segwit_optional,
-		set_shutdown_any_segwit_required);
+		set_shutdown_any_segwit_required, supports_shutdown_anysegwit, requires_shutdown_anysegwit);
+	define_feature!(45, ChannelType, [InitContext, NodeContext],
+		"Feature flags for `option_channel_type`.", set_channel_type_optional,
+		set_channel_type_required, supports_channel_type, requires_channel_type);
 	define_feature!(55, Keysend, [NodeContext],
-		"Feature flags for keysend payments.", set_keysend_optional, set_keysend_required);
+		"Feature flags for keysend payments.", set_keysend_optional, set_keysend_required,
+		supports_keysend, requires_keysend);
 
 	#[cfg(test)]
 	define_feature!(123456789, UnknownFeature, [NodeContext, ChannelContext, InvoiceContext],
 		"Feature flags for an unknown feature used in testing.", set_unknown_feature_optional,
-		set_unknown_feature_required);
+		set_unknown_feature_required, supports_unknown_test_feature, requires_unknown_test_feature);
 }
 
 /// Tracks the set of features which a node implements, templated by the context in which it
@@ -388,6 +444,18 @@ pub type ChannelFeatures = Features<sealed::ChannelContext>;
 /// Features used within an invoice.
 pub type InvoiceFeatures = Features<sealed::InvoiceContext>;
 
+/// Features used within the channel_type field in an OpenChannel message.
+///
+/// A channel is always of some known "type", describing the transaction formats used and the exact
+/// semantics of our interaction with our peer.
+///
+/// Note that because a channel is a specific type which is proposed by the opener and accepted by
+/// the counterparty, only required features are allowed here.
+///
+/// This is serialized differently from other feature types - it is not prefixed by a length, and
+/// thus must only appear inside a TLV where its length is known in advance.
+pub type ChannelTypeFeatures = Features<sealed::ChannelTypeContext>;
+
 impl InitFeatures {
 	/// Writes all features present up to, and including, 13.
 	pub(crate) fn write_up_to_13<W: Writer>(&self, w: &mut W) -> Result<(), io::Error> {
@@ -442,6 +510,28 @@ impl InvoiceFeatures {
 	}
 }
 
+impl ChannelTypeFeatures {
+	/// Constructs the implicit channel type based on the common supported types between us and our
+	/// counterparty
+	pub(crate) fn from_counterparty_init(counterparty_init: &InitFeatures) -> Self {
+		let mut ret = counterparty_init.to_context_internal();
+		// ChannelTypeFeatures must only contain required bits, so we OR the required forms of all
+		// optional bits and then AND out the optional ones.
+		for byte in ret.flags.iter_mut() {
+			*byte |= (*byte & 0b10_10_10_10) >> 1;
+			*byte &= 0b01_01_01_01;
+		}
+		ret
+	}
+
+	/// Constructs a ChannelTypeFeatures with only static_remotekey set
+	pub(crate) fn only_static_remote_key() -> Self {
+		let mut ret = Self::empty();
+		<sealed::ChannelTypeContext as sealed::StaticRemoteKey>::set_required_bit(&mut ret.flags);
+		ret
+	}
+}
+
 impl ToBase32 for InvoiceFeatures {
 	fn write_base32<W: WriteBase32>(&self, writer: &mut W) -> Result<(), <W as WriteBase32>::Err> {
 		// Explanation for the "4": the normal way to round up when dividing is to add the divisor
@@ -553,6 +643,25 @@ impl<T: sealed::Context> Features<T> {
 		&self.flags
 	}
 
+	fn write_be<W: Writer>(&self, w: &mut W) -> Result<(), io::Error> {
+		for f in self.flags.iter().rev() { // Swap back to big-endian
+			f.write(w)?;
+		}
+		Ok(())
+	}
+
+	fn from_be_bytes(mut flags: Vec<u8>) -> Features<T> {
+		flags.reverse(); // Swap to little-endian
+		Self {
+			flags,
+			mark: PhantomData,
+		}
+	}
+
+	pub(crate) fn supports_any_optional_bits(&self) -> bool {
+		self.flags.iter().any(|&byte| (byte & 0b10_10_10_10) != 0)
+	}
+
 	/// Returns true if this `Features` object contains unknown feature flags which are set as
 	/// "required".
 	pub fn requires_unknown_bits(&self) -> bool {
@@ -585,25 +694,7 @@ impl<T: sealed::Context> Features<T> {
 	}
 }
 
-impl<T: sealed::DataLossProtect> Features<T> {
-	#[cfg(test)]
-	pub(crate) fn requires_data_loss_protect(&self) -> bool {
-		<T as sealed::DataLossProtect>::requires_feature(&self.flags)
-	}
-	#[cfg(test)]
-	pub(crate) fn supports_data_loss_protect(&self) -> bool {
-		<T as sealed::DataLossProtect>::supports_feature(&self.flags)
-	}
-}
-
 impl<T: sealed::UpfrontShutdownScript> Features<T> {
-	#[cfg(test)]
-	pub(crate) fn requires_upfront_shutdown_script(&self) -> bool {
-		<T as sealed::UpfrontShutdownScript>::requires_feature(&self.flags)
-	}
-	pub(crate) fn supports_upfront_shutdown_script(&self) -> bool {
-		<T as sealed::UpfrontShutdownScript>::supports_feature(&self.flags)
-	}
 	#[cfg(test)]
 	pub(crate) fn clear_upfront_shutdown_script(mut self) -> Self {
 		<T as sealed::UpfrontShutdownScript>::clear_bits(&mut self.flags);
@@ -613,13 +704,6 @@ impl<T: sealed::UpfrontShutdownScript> Features<T> {
 
 
 impl<T: sealed::GossipQueries> Features<T> {
-	#[cfg(test)]
-	pub(crate) fn requires_gossip_queries(&self) -> bool {
-		<T as sealed::GossipQueries>::requires_feature(&self.flags)
-	}
-	pub(crate) fn supports_gossip_queries(&self) -> bool {
-		<T as sealed::GossipQueries>::supports_feature(&self.flags)
-	}
 	#[cfg(test)]
 	pub(crate) fn clear_gossip_queries(mut self) -> Self {
 		<T as sealed::GossipQueries>::clear_bits(&mut self.flags);
@@ -627,30 +711,7 @@ impl<T: sealed::GossipQueries> Features<T> {
 	}
 }
 
-impl<T: sealed::VariableLengthOnion> Features<T> {
-	#[cfg(test)]
-	pub(crate) fn requires_variable_length_onion(&self) -> bool {
-		<T as sealed::VariableLengthOnion>::requires_feature(&self.flags)
-	}
-	pub(crate) fn supports_variable_length_onion(&self) -> bool {
-		<T as sealed::VariableLengthOnion>::supports_feature(&self.flags)
-	}
-}
-
-impl<T: sealed::StaticRemoteKey> Features<T> {
-	pub(crate) fn supports_static_remote_key(&self) -> bool {
-		<T as sealed::StaticRemoteKey>::supports_feature(&self.flags)
-	}
-	#[cfg(test)]
-	pub(crate) fn requires_static_remote_key(&self) -> bool {
-		<T as sealed::StaticRemoteKey>::requires_feature(&self.flags)
-	}
-}
-
 impl<T: sealed::InitialRoutingSync> Features<T> {
-	pub(crate) fn initial_routing_sync(&self) -> bool {
-		<T as sealed::InitialRoutingSync>::supports_feature(&self.flags)
-	}
 	// We are no longer setting initial_routing_sync now that gossip_queries
 	// is enabled. This feature is ignored by a peer when gossip_queries has 
 	// been negotiated.
@@ -660,63 +721,51 @@ impl<T: sealed::InitialRoutingSync> Features<T> {
 	}
 }
 
-impl<T: sealed::PaymentSecret> Features<T> {
-	#[cfg(test)]
-	pub(crate) fn requires_payment_secret(&self) -> bool {
-		<T as sealed::PaymentSecret>::requires_feature(&self.flags)
-	}
-	/// Returns whether the `payment_secret` feature is supported.
-	pub fn supports_payment_secret(&self) -> bool {
-		<T as sealed::PaymentSecret>::supports_feature(&self.flags)
-	}
-}
-
-impl<T: sealed::BasicMPP> Features<T> {
-	#[cfg(test)]
-	pub(crate) fn requires_basic_mpp(&self) -> bool {
-		<T as sealed::BasicMPP>::requires_feature(&self.flags)
-	}
-	// We currently never test for this since we don't actually *generate* multipath routes.
-	pub(crate) fn supports_basic_mpp(&self) -> bool {
-		<T as sealed::BasicMPP>::supports_feature(&self.flags)
-	}
-}
-
 impl<T: sealed::ShutdownAnySegwit> Features<T> {
-	pub(crate) fn supports_shutdown_anysegwit(&self) -> bool {
-		<T as sealed::ShutdownAnySegwit>::supports_feature(&self.flags)
-	}
 	#[cfg(test)]
 	pub(crate) fn clear_shutdown_anysegwit(mut self) -> Self {
 		<T as sealed::ShutdownAnySegwit>::clear_bits(&mut self.flags);
 		self
 	}
 }
-
-impl<T: sealed::Context> Writeable for Features<T> {
-	fn write<W: Writer>(&self, w: &mut W) -> Result<(), io::Error> {
-		(self.flags.len() as u16).write(w)?;
-		for f in self.flags.iter().rev() { // Swap back to big-endian
-			f.write(w)?;
+macro_rules! impl_feature_len_prefixed_write {
+	($features: ident) => {
+		impl Writeable for $features {
+			fn write<W: Writer>(&self, w: &mut W) -> Result<(), io::Error> {
+				(self.flags.len() as u16).write(w)?;
+				self.write_be(w)
+			}
+		}
+		impl Readable for $features {
+			fn read<R: io::Read>(r: &mut R) -> Result<Self, DecodeError> {
+				Ok(Self::from_be_bytes(Vec::<u8>::read(r)?))
+			}
 		}
-		Ok(())
 	}
 }
+impl_feature_len_prefixed_write!(InitFeatures);
+impl_feature_len_prefixed_write!(ChannelFeatures);
+impl_feature_len_prefixed_write!(NodeFeatures);
+impl_feature_len_prefixed_write!(InvoiceFeatures);
 
-impl<T: sealed::Context> Readable for Features<T> {
+// Because ChannelTypeFeatures only appears inside of TLVs, it doesn't have a length prefix when
+// serialized. Thus, we can't use `impl_feature_len_prefixed_write`, above, and have to write our
+// own serialization.
+impl Writeable for ChannelTypeFeatures {
+	fn write<W: Writer>(&self, w: &mut W) -> Result<(), io::Error> {
+		self.write_be(w)
+	}
+}
+impl Readable for ChannelTypeFeatures {
 	fn read<R: io::Read>(r: &mut R) -> Result<Self, DecodeError> {
-		let mut flags: Vec<u8> = Readable::read(r)?;
-		flags.reverse(); // Swap to little-endian
-		Ok(Self {
-			flags,
-			mark: PhantomData,
-		})
+		let v = io_extras::read_to_end(r)?;
+		Ok(Self::from_be_bytes(v))
 	}
 }
 
 #[cfg(test)]
 mod tests {
-	use super::{ChannelFeatures, InitFeatures, InvoiceFeatures, NodeFeatures};
+	use super::{ChannelFeatures, ChannelTypeFeatures, InitFeatures, InvoiceFeatures, NodeFeatures};
 	use bitcoin::bech32::{Base32Len, FromBase32, ToBase32, u5};
 
 	#[test]
@@ -769,6 +818,11 @@ mod tests {
 		assert!(!NodeFeatures::known().requires_basic_mpp());
 		assert!(!InvoiceFeatures::known().requires_basic_mpp());
 
+		assert!(InitFeatures::known().supports_channel_type());
+		assert!(NodeFeatures::known().supports_channel_type());
+		assert!(!InitFeatures::known().requires_channel_type());
+		assert!(!NodeFeatures::known().requires_channel_type());
+
 		assert!(InitFeatures::known().supports_shutdown_anysegwit());
 		assert!(NodeFeatures::known().supports_shutdown_anysegwit());
 
@@ -807,11 +861,15 @@ mod tests {
 			// - var_onion_optin (req) | static_remote_key (req) | payment_secret(req)
 			// - basic_mpp
 			// - opt_shutdown_anysegwit
-			assert_eq!(node_features.flags.len(), 4);
+			// -
+			// - option_channel_type
+			assert_eq!(node_features.flags.len(), 6);
 			assert_eq!(node_features.flags[0], 0b00000010);
 			assert_eq!(node_features.flags[1], 0b01010001);
 			assert_eq!(node_features.flags[2], 0b00000010);
 			assert_eq!(node_features.flags[3], 0b00001000);
+			assert_eq!(node_features.flags[4], 0b00000000);
+			assert_eq!(node_features.flags[5], 0b00100000);
 		}
 
 		// Check that cleared flags are kept blank when converting back:
@@ -875,4 +933,15 @@ mod tests {
 		let features_deserialized = InvoiceFeatures::from_base32(&features_as_u5s).unwrap();
 		assert_eq!(features, features_deserialized);
 	}
+
+	#[test]
+	fn test_channel_type_mapping() {
+		// If we map an InvoiceFeatures with StaticRemoteKey optional, it should map into a
+		// required-StaticRemoteKey ChannelTypeFeatures.
+		let init_features = InitFeatures::empty().set_static_remote_key_optional();
+		let converted_features = ChannelTypeFeatures::from_counterparty_init(&init_features);
+		assert_eq!(converted_features, ChannelTypeFeatures::only_static_remote_key());
+		assert!(!converted_features.supports_any_optional_bits());
+		assert!(converted_features.requires_static_remote_key());
+	}
 }
