@@ -17,11 +17,13 @@
 use core::cmp;
 use core::fmt;
 
-static LOG_LEVEL_NAMES: [&'static str; 5] = ["TRACE", "DEBUG", "INFO", "WARN", "ERROR"];
+static LOG_LEVEL_NAMES: [&'static str; 6] = ["GOSSIP", "TRACE", "DEBUG", "INFO", "WARN", "ERROR"];
 
 /// An enum representing the available verbosity levels of the logger.
 #[derive(Copy, Clone, PartialEq, Eq, Debug, Hash)]
 pub enum Level {
+	/// Designates extremely verbose information, including gossip-induced messages
+	Gossip,
 	/// Designates very low priority, often extremely verbose, information
 	Trace,
 	/// Designates lower priority information
@@ -78,38 +80,51 @@ impl Level {
 	/// Returns the most verbose logging level.
 	#[inline]
 	pub fn max() -> Level {
-		Level::Trace
+		Level::Gossip
 	}
 }
 
 /// A Record, unit of logging output with Metadata to enable filtering
 /// Module_path, file, line to inform on log's source
-/// (C-not exported) - we convert to a const char* instead
-#[derive(Clone,Debug)]
+#[derive(Clone, Debug)]
 pub struct Record<'a> {
 	/// The verbosity level of the message.
 	pub level: Level,
+	#[cfg(not(c_bindings))]
 	/// The message body.
 	pub args: fmt::Arguments<'a>,
+	#[cfg(c_bindings)]
+	/// The message body.
+	pub args: String,
 	/// The module path of the message.
-	pub module_path: &'a str,
+	pub module_path: &'static str,
 	/// The source file containing the message.
-	pub file: &'a str,
+	pub file: &'static str,
 	/// The line containing the message.
 	pub line: u32,
+
+	#[cfg(c_bindings)]
+	/// We don't actually use the lifetime parameter in C bindings (as there is no good way to
+	/// communicate a lifetime to a C, or worse, Java user).
+	_phantom: core::marker::PhantomData<&'a ()>,
 }
 
 impl<'a> Record<'a> {
 	/// Returns a new Record.
 	/// (C-not exported) as fmt can't be used in C
 	#[inline]
-	pub fn new(level: Level, args: fmt::Arguments<'a>, module_path: &'a str, file: &'a str, line: u32) -> Record<'a> {
+	pub fn new(level: Level, args: fmt::Arguments<'a>, module_path: &'static str, file: &'static str, line: u32) -> Record<'a> {
 		Record {
 			level,
+			#[cfg(not(c_bindings))]
 			args,
+			#[cfg(c_bindings)]
+			args: format!("{}", args),
 			module_path,
 			file,
-			line
+			line,
+			#[cfg(c_bindings)]
+			_phantom: core::marker::PhantomData,
 		}
 	}
 }
@@ -163,13 +178,14 @@ mod tests {
 			log_info!(self.logger, "This is an info");
 			log_debug!(self.logger, "This is a debug");
 			log_trace!(self.logger, "This is a trace");
+			log_gossip!(self.logger, "This is a gossip");
 		}
 	}
 
 	#[test]
 	fn test_logging_macros() {
 		let mut logger = TestLogger::new();
-		logger.enable(Level::Trace);
+		logger.enable(Level::Gossip);
 		let logger : Arc<Logger> = Arc::new(logger);
 		let wrapper = WrapperLog::new(Arc::clone(&logger));
 		wrapper.call_macros();
@@ -189,7 +205,10 @@ mod tests {
 		assert!(Level::Debug > Level::Trace);
 		assert!(Level::Debug >= Level::Trace);
 		assert!(Level::Debug >= Level::Debug);
+		assert!(Level::Trace > Level::Gossip);
+		assert!(Level::Trace >= Level::Gossip);
 		assert!(Level::Trace >= Level::Trace);
+		assert!(Level::Gossip >= Level::Gossip);
 
 		assert!(Level::Error <= Level::Error);
 		assert!(Level::Warn < Level::Error);
@@ -204,5 +223,8 @@ mod tests {
 		assert!(Level::Trace < Level::Debug);
 		assert!(Level::Trace <= Level::Debug);
 		assert!(Level::Trace <= Level::Trace);
+		assert!(Level::Gossip < Level::Trace);
+		assert!(Level::Gossip <= Level::Trace);
+		assert!(Level::Gossip <= Level::Gossip);
 	}
 }
